==> cell-tracking-bc-2023.9.tar.gz: cell-tracking-bc-2023.9/cell_tracking_BC/catalog/processing/registration.py <==
# Copyright CNRS/Inria/UCA
# Contributor(s): Eric Debreuve (since 2021)
#
# eric.debreuve@cnrs.fr
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from typing import Dict, Sequence

import numpy as nmpy
from skimage import registration as rgst
from skimage import transform as trsf

array_t = nmpy.ndarray


def RegisteredInTranslation(
    unregistered: array_t,
    /,
    *,
    # Keyword-only as part of coding conventions
    channels: Dict[str, array_t] = None,
    reference: str = None,
) -> array_t:
    """"""
    reference_frame = channels[reference]
    if unregistered is reference_frame:
        return unregistered

    translation = TranslationFromTo(reference_frame, unregistered)

    return WithTranslationApplied(unregistered, translation)


def TranslationFromTo(reference: array_t, unregistered: array_t, /) -> array_t:
    """"""
    return rgst.phase_cross_correlation(
        reference,
        unregistered,
        upsample_factor=8,
        return_error=False,
    )


def WithTranslationApplied(
    unregistered: array_t, translation: Sequence[int], /
) -> array_t:
    """"""
    if nmpy.any(translation != 0.0):
        translation = trsf.EuclideanTransform(translation=translation)
        output = trsf.warp(unregistered, translation, preserve_range=True)
        if ("bool" in unregistered.dtype.name) or ("int" in unregistered.dtype.name):
            output = nmpy.around(output).astype(unregistered.dtype)
        else:
            output = output.astype(unregistered.dtype, copy=False)
    else:
        output = unregistered

    return output

==> cell-tracking-bc-2023.15.tar.gz: cell-tracking-bc-2023.15/cell_tracking_BC/catalog/processing/registration.py <==
# Copyright CNRS/Inria/UCA
# Contributor(s): Eric Debreuve (since 2021)
#
# eric.debreuve@cnrs.fr
#
# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from typing import Dict, Sequence

import numpy as nmpy
from skimage import registration as rgst
from skimage import transform as trsf

array_t = nmpy.ndarray


def RegisteredInTranslation(
    unregistered: array_t,
    /,
    *,
    # Keyword-only as part of coding conventions
    channels: Dict[str, array_t] = None,
    reference: str = None,
) -> array_t:
    """"""
    reference_frame = channels[reference]
    if unregistered is reference_frame:
        return unregistered

    translation = TranslationFromTo(reference_frame, unregistered)

    return WithTranslationApplied(unregistered, translation)


def TranslationFromTo(reference: array_t, unregistered: array_t, /) -> array_t:
    """"""
    output, *_ = rgst.phase_cross_correlation(
        reference,
        unregistered,
        upsample_factor=8,
    )

    return output


def WithTranslationApplied(
    unregistered: array_t, translation: Sequence[int], /
) -> array_t:
    """"""
    if nmpy.any(translation != 0.0):
        translation = trsf.EuclideanTransform(translation=translation)
        output = trsf.warp(unregistered, translation, preserve_range=True)
        if ("bool" in unregistered.dtype.name) or ("int" in unregistered.dtype.name):
            output = nmpy.around(output).astype(unregistered.dtype)
        else:
            output = output.astype(unregistered.dtype, copy=False)
    else:
        output = unregistered

    return output
